@@ -10,8 +10,8 @@
   {
     RFver <- read.dcf(file = system.file("DESCRIPTION", package = pkgname), fields = "Version")
     mess <- paste(pkgname, RFver, "loaded.\n")
-    mess <- paste(mess, "/!\\ Be careful : note that some BigBoss options have been changed between biomod2 v4.2-6 and previous versions./!\\ 
-                  \n\n Please welcome a new model : RFd. This is a random forest model with a automatic down sampling to make your life easier.\n")
+    mess <- paste(mess, "/!\\ Welcome to augmented biomod2 with abundance modeling available! (*o*)\n",
+                  "Take a look at the HOME and NEWS section on the website to see all the features!\n")
     packageStartupMessage(mess)
     
     toLoad <- unique(ModelsTable$package[-which(ModelsTable$package %in% c("MAXENT", "biomod2"))])
