@@ -1,6 +1,6 @@
 ###################################################################################################
 ##' @name bm_PlotEvalBoxplot
-##' @author Damien Georges, Maya Gueguen
+##' @author Damien Georges, Maya Guéguen
 ##' 
 ##' @title Plot boxplot of evaluation scores
 ##' 
@@ -87,10 +87,10 @@
 ##' } else {
 ##' 
 ##'   # Format Data with true absences
-##'   myBiomodData <- BIOMOD_FormatingData(resp.var = myResp,
-##'                                        expl.var = myExpl,
+##'   myBiomodData <- BIOMOD_FormatingData(resp.name = myRespName,
+##'                                        resp.var = myResp,
 ##'                                        resp.xy = myRespXY,
-##'                                        resp.name = myRespName)
+##'                                        expl.var = myExpl)
 ##' 
 ##'   # Model single models
 ##'   myBiomodModelOut <- BIOMOD_Modeling(bm.format = myBiomodData,
@@ -100,7 +100,7 @@
 ##'                                       CV.nb.rep = 2,
 ##'                                       CV.perc = 0.8,
 ##'                                       OPT.strategy = 'bigboss',
-##'                                       metric.eval = c('TSS','ROC'),
+##'                                       metric.eval = c('TSS', 'ROC'),
 ##'                                       var.import = 3,
 ##'                                       seed.val = 42)
 ##' }
@@ -114,8 +114,9 @@
 ##' bm_PlotEvalBoxplot(bm.out = myBiomodModelOut, group.by = c('algo', 'run'))
 ##' 
 ##' 
-##' @importFrom ggplot2 ggplot aes_string geom_boxplot facet_wrap xlab 
+##' @importFrom ggplot2 ggplot geom_boxplot facet_wrap xlab 
 ##' theme element_blank element_rect element_text labs
+##' @importFrom rlang .data
 ##' 
 ##' @export
 ##' 
@@ -134,17 +135,19 @@ bm_PlotEvalBoxplot <- function(bm.out, dataset = 'calibration', group.by = c('al
   ## 1. Get data for graphic ----------------------------------------------------------------------
   ## Get evaluation values
   scores <- get_evaluations(bm.out)
-  
-  if (!is.null(scores))
-  {
-    ## Prepare data table for graphic
+  if (!is.null(scores) && any(!is.na(scores[, dataset]))) {
+  ## Prepare data table for graphic
     ggdat = scores
     
     ## 2. PLOT graphic ------------------------------------------------------------------------------
-    gg <- ggplot(ggdat, aes_string(x = group.by[1], y = dataset, fill = group.by[2])) +
+    gg <- ggplot(ggdat, aes(x = .data[[group.by[1]]], y = .data[[dataset]], color = .data[[group.by[2]]])) +
       geom_boxplot() + ## add boxplot
       facet_wrap("metric.eval", scales = scales) +
-      xlab("") +
+      labs(x = "", y = ""
+           , subtitle = switch(dataset
+                               , "calibration" = "Calibration dataset"
+                               , "validation" = "Validation dataset"
+                               , "evaluation" = "Evaluation dataset")) +
       theme(legend.title = element_blank()
             , legend.key = element_rect(fill = "white")
             , axis.text.x = element_text(angle = 45, hjust = 1))
@@ -179,7 +182,7 @@ bm_PlotEvalBoxplot <- function(bm.out, dataset = 'calibration', group.by = c('al
     }
   } else if (inherits(bm.out, "BIOMOD.ensemble.models.out")) {
     for (i in 1:length(group.by)) {
-      .fun_testIfIn(TRUE, paste0("group.by[", i, "]"), group.by[i], c("full.name", "merged.by.PA", "merged.by.run", "algo"))
+      .fun_testIfIn(TRUE, paste0("group.by[", i, "]"), group.by[i], c("full.name", "merged.by.PA", "merged.by.run", "algo", "filtered.by"))
     }
   } 
   
@@ -188,7 +191,11 @@ bm_PlotEvalBoxplot <- function(bm.out, dataset = 'calibration', group.by = c('al
   if ("scales" %in% names(args)) {
     .fun_testIfIn(TRUE, "args$scales", args$scales, c('fixed', 'free_x', 'free_y', 'free'))
   } else {
-    args$scales = "fixed"
+    if(bm.out@data.type == "binary"){
+      args$scales = "fixed"
+    } else {
+      args$scales = "free_y"
+    }
   }
   
   
